// Arduino Ethernet Light PWM
//
// George Smart M1GEO
//
// Monday 21 March 2015
//
// http://www.george-smart.co.uk/wiki/Editing_Arduino_Ethernet_Light_PWM

#define OUTA  5
#define OUTB  6

#include <SPI.h>
#include <Ethernet.h>

// MAC address from Ethernet shield sticker under board
byte mac[] = {0xB7, 0x17, 0x07, 0xFF, 0xCB, 0x43}; // random MAC address
IPAddress ip(192,168,1,188);  // my IP
IPAddress gw(192,168,1,254);  // gateway
IPAddress ns(192,168,1,254);  // nameserver
IPAddress sn(255,255,255,0);  // subnet mask

EthernetServer server(80);  // create a server at port 80

String HTTP_req;          // stores the HTTP request
byte LED_Val = 128;   // state of LED, off by default

void setup()
{
    Serial.begin(115200);       // for diagnostics
    Serial.println("Waiting for ETH to stabilise");
    delay(5000);
    Serial.println("Setting up");
    Ethernet.begin(mac, ip, ns, gw, sn);  // initialize Ethernet device
    Serial.print("MY IP: ");
    Serial.println(Ethernet.localIP());
    server.begin();           // start to listen for clients
    pinMode(OUTA, OUTPUT);       // LED on pin 5
    pinMode(OUTB, OUTPUT);       // LED on pin 6
}

void loop()
{
    EthernetClient client = server.available();  // try to get client

    if (client) {  // got client?
        boolean currentLineIsBlank = true;
        while (client.connected()) {
            if (client.available()) {   // client data available to read
                char c = client.read(); // read 1 byte (character) from client
                HTTP_req += c;  // save the HTTP request 1 char at a time
                // last line of client request is blank and ends with \n
                // respond to client only after last line received
                if (c == '\n' && currentLineIsBlank) {
                    // send a standard http response header
                    client.println("HTTP/1.1 200 OK");
                    client.println("Content-Type: text/html");
                    client.println("Connection: close");
                    client.println();

                    // send web page
                    client.println("<html>");
                    client.println("<!-- Code Generated on the fly by Arduino Uno. George Smart M1GEO -->");
                    client.println("<head>");
                    
                    // disable caching if possible
                    client.println("<meta http-equiv=\"cache-control\" content=\"max-age=0\" />");
                    client.println("<meta http-equiv=\"cache-control\" content=\"no-cache\" />");
                    client.println("<meta http-equiv=\"expires\" content=\"0\" />");
                    client.println("<meta http-equiv=\"expires\" content=\"Tue, 01 Jan 1980 1:00:00 GMT\" />");
                    client.println("<meta http-equiv=\"pragma\" content=\"no-cache\" />");
                    
                    // direct the browser elsewhere for the favicon
                    client.println("<link rel=\"shortcut icon\" type=\"image/x-icon\" href=\"http://www.george-smart.co.uk/favicon.ico\" />");

                    // process
                    ProcessValue();

                    client.println("<title>GS Lights</title>");
                    client.println("</head>");
                    client.println("<body>");
                    client.println("<h1>George Bedroom Lights</h1><br>");
                    client.println("<form action=\"/\">");
                    client.println("<p>");
                    client.print  ("  Enter Brightness: <input type=\"number\" name=\"v\" min=\"0\" max=\"255\" value=\"");
                    client.print  (LED_Val); //insert current value
                    client.print  ("\" length> <em>(0-255)</em><br>");
                    client.println("</p>");
                    client.println("<p><input type=\"submit\" value=\"Update\"></p>");
                    client.println("</form>");
                    client.println("</body>");
                    client.println("</html>");
                    Serial.print(HTTP_req);
                    HTTP_req = "";    // finished with request, empty string
                    break;
                }
                // every line of text received from the client ends with \r\n
                if (c == '\n') {
                    // last character on line of received text
                    // starting new line with next character read
                    currentLineIsBlank = true;
                } 
                else if (c != '\r') {
                    // a text character was received from client
                    currentLineIsBlank = false;
                }
            } // end if (client.available())
        } // end while (client.connected())
        delay(1);      // give the web browser time to receive the data
        client.stop(); // close the connection
        Serial.println("Disconnected...");
    } // end if (client)
}

// switch LED and send back HTML for LED checkbox
void ProcessValue()
{
    int start = -1;
    int stops = -1;
    char a = 0;
    String numasstr;
    int n;

    start = HTTP_req.indexOf("?v=");
       
    if (start != -1) {
      n = start + 3; // (9 because "?v=")
      while (a != ' ') {
        a = HTTP_req.charAt(n);
        numasstr += a;
        n++;
      }

      LED_Val = numasstr.toInt();

      Serial.print  ("***** New Value: '");
      Serial.print  (LED_Val);
      Serial.println("'");
      
      if (LED_Val == 0) {
        digitalWrite(OUTA, LOW);
        digitalWrite(OUTB, LOW);
      } else { 
        if (LED_Val == 255) {
          digitalWrite(OUTA, HIGH);
          digitalWrite(OUTB, HIGH);
        } else {
          analogWrite(OUTA, LED_Val);
          analogWrite(OUTB, LED_Val);
        }
      }
      
      Serial.println("Updated.");       
    }
}
